@@ -119,96 +119,55 @@ test_that("get_cluster() works for rma.mv objects.", {
   expect_equal(get_cluster(mod_F), study_fac, tolerance = tol)
 })
 
-test_that("get_boot_F() works for rma.mv objects.", {
+test_that("get_boot_F_F() works for rma.mv objects.", {
 
   y_boot <- oswald2013$yi
 
   Cmat_int <- constrain_zero(1, coefs = coef(mod_A))
 
+  f_A <- get_boot_F_f(mod_A, C_mat = Cmat_int)
   expect_equal(
-    get_boot_F(mod_A, y_boot = y_boot, C_mat = Cmat_int, cluster = get_cluster(mod_A)),
+    f_A(y_boot = y_boot, cluster = get_cluster(mod_A)),
     Wald_test(mod_A, constraints = Cmat_int, vcov = "CR0", test = "Naive-F")$Fstat, tolerance = tol
   )
 
+  f_B <- get_boot_F_f(mod_B, C_mat = Cmat_A, type = "CR1", test = "HTZ")
   expect_equal(
-    get_boot_F(mod_B, y_boot = y_boot, C_mat = Cmat_A, cluster = get_cluster(mod_B),
-               type = "CR1", test = "HTZ"),
+    f_B(y_boot = y_boot, cluster = get_cluster(mod_B)),
     Wald_test(mod_B, constraints = Cmat_A, vcov = "CR1", test = "HTZ")$Fstat, tolerance = tol
   )
 
+  f_C1 <- get_boot_F_f(mod_C1, C_mat = Cmat_B, type = "CR2", test = "EDT")
   expect_equal(
-    get_boot_F(mod_C1, y_boot = y_boot, C_mat = Cmat_B, cluster = get_cluster(mod_C1),
-               type = "CR2", test = "EDT"),
+    f_C1(y_boot = y_boot, cluster = get_cluster(mod_C1)),
     Wald_test(mod_C1, constraints = Cmat_B, vcov = "CR2", test = "EDT")$Fstat, tolerance = tol
   )
 
+  f_C1 <- get_boot_F_f(mod_C1, C_mat = Cmat_D, type = "CR3", test = "chi-sq")
   expect_equal(
-    get_boot_F(mod_C1, y_boot = y_boot, C_mat = Cmat_D, cluster = get_cluster(mod_C1),
-               type = "CR3", test = "chi-sq"),
+    f_C1(y_boot = y_boot, cluster = get_cluster(mod_C1)),
     Wald_test(mod_C1, constraints = Cmat_D, vcov = "CR3", test = "chi-sq")$Fstat, tolerance = tol
   )
 
+  f_C2 <- get_boot_F_f(mod_C2, C_mat = Cmat_E, type = "CR2", test = "Naive-Fp")
   expect_equal(
-    get_boot_F(mod_C2, y_boot = y_boot, C_mat = Cmat_E, cluster = get_cluster(mod_C2),
-               type = "CR2", test = "Naive-Fp"),
+    f_C2(y_boot = y_boot, cluster = get_cluster(mod_C2)),
     Wald_test(mod_C2, constraints = Cmat_E, vcov = "CR2", test = "Naive-Fp")$Fstat, tolerance = tol
   )
 
+  f_C2 <- get_boot_F_f(mod_C2, C_mat = Cmat_F, type = "CR0", test = "HTA")
   expect_equal(
-    get_boot_F(mod_C2, y_boot = y_boot, C_mat = Cmat_F, cluster = get_cluster(mod_C2),
-               type = "CR0", test = "HTA"),
+    f_C2(y_boot = y_boot, cluster = get_cluster(mod_C2)),
     Wald_test(mod_C2, constraints = Cmat_F, vcov = "CR0", test = "HTA")$Fstat, tolerance = tol
   )
 
+  f_Y <- get_boot_F_f(mod_Y, C_mat = Cmat_F, type = "CR0", test = "HTA")
   expect_equal(
-    get_boot_F(mod_Y, y_boot = y_boot, C_mat = Cmat_F, cluster = oswald2013$Study,
-               type = "CR0", test = "HTA"),
+    f_Y(y_boot = y_boot, cluster = oswald2013$Study),
     Wald_test(mod_Y, constraints = Cmat_F, cluster = oswald2013$Study,
               vcov = "CR0", test = "HTA")$Fstat, tolerance = tol
   )
 
-  # get_boot_F_f
-
-  expect_equal(
-    get_boot_F(mod_A, y_boot = y_boot, C_mat = Cmat_int, cluster = get_cluster(mod_A)),
-    get_boot_F_f(mod_A, C_mat = Cmat_int, cluster = get_cluster(mod_A))(y_boot), tolerance = tol
-  )
-
-  expect_equal(
-    get_boot_F(mod_B, y_boot = y_boot, C_mat = Cmat_A, cluster = get_cluster(mod_B),
-               type = "CR1", test = "HTZ"),
-    get_boot_F_f(mod_B, C_mat = Cmat_A, cluster = get_cluster(mod_B),
-               type = "CR1", test = "HTZ")(y_boot), tolerance = tol
-  )
-
-  expect_equal(
-    get_boot_F(mod_C1, y_boot = y_boot, C_mat = Cmat_B, cluster = get_cluster(mod_C1),
-               type = "CR2", test = "EDT"),
-    get_boot_F_f(mod_C1, C_mat = Cmat_B, cluster = get_cluster(mod_C1),
-               type = "CR2", test = "EDT")(y_boot), tolerance = tol
-  )
-
-  expect_equal(
-    get_boot_F(mod_C1, y_boot = y_boot, C_mat = Cmat_D, cluster = get_cluster(mod_C1),
-               type = "CR3", test = "chi-sq"),
-    get_boot_F_f(mod_C1, C_mat = Cmat_D, cluster = get_cluster(mod_C1),
-               type = "CR3", test = "chi-sq")(y_boot), tolerance = tol
-  )
-
-  expect_equal(
-    get_boot_F(mod_C2, y_boot = y_boot, C_mat = Cmat_E, cluster = get_cluster(mod_C2),
-               type = "CR2", test = "Naive-Fp"),
-    get_boot_F_f(mod_C2, C_mat = Cmat_E, cluster = get_cluster(mod_C2),
-               type = "CR2", test = "Naive-Fp")(y_boot), tolerance = tol
-  )
-
-  expect_equal(
-    get_boot_F(mod_C2, y_boot = y_boot, C_mat = Cmat_F, cluster = get_cluster(mod_C2),
-               type = "CR0", test = "HTA"),
-    get_boot_F_f(mod_C2, C_mat = Cmat_F, cluster = get_cluster(mod_C2),
-               type = "CR0", test = "HTA")(y_boot), tolerance = tol
-  )
-
 
   oswald2013$yi_boot <- oswald2013$yi + rnorm(nrow(oswald2013))
 
@@ -226,10 +185,8 @@ test_that("get_boot_F() works for rma.mv objects.", {
 
   Cmat_G <- constrain_equal("Crit.Cat", reg_ex = TRUE, coefs = coef(mod_G))
   F_stat <- Wald_test(mod_F, constraints = Cmat_G, vcov = "CR0", test = "HTZ")$Fstat
-  G_stat <- get_boot_F(mod_G, y_boot = y_boot, C_mat = Cmat_G, cluster = get_cluster(mod_G),
-                       type = "CR0", test = "HTZ")
-  H_stat <- get_boot_F(mod_H, y_boot = y_boot, C_mat = Cmat_G, cluster = get_cluster(mod_G),
-                       type = "CR0", test = "HTZ")
+  G_stat <- get_boot_F_f(mod_G, C_mat = Cmat_G, type = "CR0", test = "HTZ")(y_boot = y_boot, cluster = get_cluster(mod_G))
+  H_stat <- get_boot_F_f(mod_H, C_mat = Cmat_G, type = "CR0", test = "HTZ")(y_boot = y_boot, cluster = get_cluster(mod_G))
 
   expect_equal(G_stat, F_stat, tolerance = tol)
 
@@ -272,6 +229,7 @@ compare_mod_results <- function(mod, scram, ord = 1:length(get_res(mod)), tol =
   expect_equal(get_cluster(mod)[ord], get_cluster(scram), tolerance = tol)
 }
 
+
 test_that("Wald_test_cwb() results do not depend on sort order.", {
 
   skip_on_cran()
@@ -283,6 +241,7 @@ test_that("Wald_test_cwb() results do not depend on sort order.", {
 
   V_scram <- impute_covariance_matrix(vi = oswald_scram$vi, cluster = oswald_scram$Study, r = 0.4)
 
+
   scram_A <- rma.mv(yi = yi, V = V_scram,
                     random = ~ 1 | Study / esID,
                     data = oswald_scram,
@@ -389,47 +348,101 @@ test_that("Wald_test_cwb() results do not depend on sort order.", {
 
 test_that("Wald_test_cwb() works when rma.mv uses subset.", {
 
-  oswald_sub <- subset(oswald2013_full, Crit.Cat == "Microbehavior")
+  set.seed(20230212)
+  oswald_missX <- oswald2013_full
+  oswald_missX$IAT.Focus[sample(1:nrow(oswald2013_full), size = 40)] <- NA
+  oswald_sub <- subset(oswald_missX, Crit.Cat == "Microbehavior")
+  oswald_complete <- subset(oswald_missX, Crit.Cat == "Microbehavior" & !is.na(IAT.Focus))
 
-  V_full <- impute_covariance_matrix(vi = oswald2013_full$vi, cluster = oswald2013_full$Study, r = 0.4)
+  V_missX <- impute_covariance_matrix(vi = oswald_missX$vi, cluster = oswald_missX$Study, r = 0.4)
   V_sub <- impute_covariance_matrix(vi = oswald_sub$vi, cluster = oswald_sub$Study, r = 0.4)
+  V_complete <- impute_covariance_matrix(vi = oswald_complete$vi, cluster = oswald_complete$Study, r = 0.4)
 
+  mod_complete <- rma.mv(yi = yi, V = V_complete,
+                         mods = ~ 0 + IAT.Focus + Crit.Domain,
+                         random = ~ 1 | Study / esID,
+                         data = oswald_complete,
+                         sparse = TRUE)
 
-  mod_full <- rma.mv(yi = yi, V = V_sub,
-                     mods = ~ 0 + IAT.Focus + Crit.Domain,
-                     random = ~ 1 | Study / esID,
-                     data = oswald_sub,
-                     sparse = TRUE)
+  expect_warning(
+    mod_sub <- rma.mv(yi = yi, V = V_sub,
+                      mods = ~ 0 + IAT.Focus + Crit.Domain,
+                      random = ~ 1 | Study / esID,
+                      data = oswald_sub,
+                      sparse = TRUE)
+  )
 
-  mod_sub <- rma.mv(yi = yi, V = V_full,
-                    mods = ~ 0 + IAT.Focus + Crit.Domain,
-                    random = ~ 1 | Study / esID,
-                    data = oswald2013_full,
-                    subset = Crit.Cat == "Microbehavior",
-                    sparse = TRUE)
+  expect_warning(
+    mod_missX <- rma.mv(yi = yi, V = V_missX,
+                        mods = ~ 0 + IAT.Focus + Crit.Domain,
+                        random = ~ 1 | Study / esID,
+                        data = oswald_missX,
+                        subset = Crit.Cat == "Microbehavior",
+                        sparse = TRUE)
+  )
 
-  compare_mod_results(mod_full, mod_sub)
+  compare_mod_results(mod_complete, mod_sub)
+  compare_mod_results(mod_complete, mod_missX)
 
-  test_full <- Wald_test_cwb(mod_full,
-                             constraints = constrain_equal("IAT.Focus", reg_ex = TRUE),
-                             R = 3,
-                             auxiliary_dist = "Rademacher",
-                             adjust = "CR0",
-                             type = "CR0",
-                             test = "Naive-F",
-                             seed = 19)
+  test_complete <- Wald_test_cwb(mod_sub,
+                                 constraints = constrain_equal("IAT.Focus", reg_ex = TRUE),
+                                 R = 5,
+                                 auxiliary_dist = "Rademacher",
+                                 adjust = "CR0",
+                                 type = "CR0",
+                                 test = "Naive-F",
+                                 seed = 19)
 
   test_sub <- Wald_test_cwb(mod_sub,
                             constraints = constrain_equal("IAT.Focus", reg_ex = TRUE),
-                            R = 3,
+                            R = 5,
                             auxiliary_dist = "Rademacher",
                             adjust = "CR0",
                             type = "CR0",
                             test = "Naive-F",
                             seed = 19)
 
-  expect_equal(attr(test_full, "original"), attr(test_sub, "original"), tolerance = tol)
-  expect_equal(attr(test_full, "bootstraps"), attr(test_sub, "bootstraps"), tolerance = tol)
+  expect_equal(attr(test_complete, "original"), attr(test_sub, "original"), tolerance = tol)
+  expect_equal(attr(test_complete, "bootstraps"), attr(test_sub, "bootstraps"), tolerance = tol)
+
+  test_missX <- Wald_test_cwb(mod_missX,
+                              constraints = constrain_equal("IAT.Focus", reg_ex = TRUE),
+                              R = 5,
+                              auxiliary_dist = "Rademacher",
+                              adjust = "CR0",
+                              type = "CR0",
+                              test = "Naive-F",
+                              seed = 19)
+
+  expect_equal(attr(test_complete, "original"), attr(test_missX, "original"), tolerance = tol)
+  expect_equal(attr(test_complete, "bootstraps"), attr(test_missX, "bootstraps"), tolerance = tol)
+
+  test_missX2 <- Wald_test_cwb(mod_missX,
+                               constraints = constrain_equal("IAT.Focus", reg_ex = TRUE),
+                               cluster = oswald_missX$Study,
+                               R = 5,
+                               auxiliary_dist = "Rademacher",
+                               adjust = "CR0",
+                               type = "CR0",
+                               test = "Naive-F",
+                               seed = 19)
+
+  expect_equal(attr(test_missX2, "original"), attr(test_missX, "original"), tolerance = tol)
+  expect_equal(attr(test_missX2, "bootstraps"), attr(test_missX, "bootstraps"), tolerance = tol)
+
+  test_sub2 <- Wald_test_cwb(mod_sub,
+                             constraints = constrain_equal("IAT.Focus", reg_ex = TRUE),
+                             cluster = oswald_sub$Study,
+                             R = 5,
+                             auxiliary_dist = "Rademacher",
+                             adjust = "CR0",
+                             type = "CR0",
+                             test = "Naive-F",
+                             seed = 19)
+
+  expect_equal(attr(test_sub2, "original"), attr(test_sub, "original"), tolerance = tol)
+  expect_equal(attr(test_sub2, "bootstraps"), attr(test_sub, "bootstraps"), tolerance = tol)
+
 
 })
 
@@ -440,6 +453,7 @@ test_that("Wald_test_cwb() works with user-weighted rma.mv models.", {
   skip_if(packageVersion("clubSandwich") < '0.5.5.9999')
 
   W_mat <- impute_covariance_matrix(vi = oswald2013$wt, cluster = oswald2013$Study, r = 0, return_list = FALSE)
+
   mod_wt1 <- rma.mv(yi ~ 0 + Crit.Cat + Crit.Domain + IAT.Focus + Scoring,
                      V = V, W = W_mat,
                      random = ~ 1 | Study,
